package com.qmth.sop.task.job;

import com.qmth.sop.business.bean.result.FlowTaskSmsResult;
import com.qmth.sop.business.entity.TSJobRemind;
import com.qmth.sop.business.service.TSJobRemindService;
import com.qmth.sop.business.util.SmsSendUtil;
import com.qmth.sop.common.contant.SystemConstant;
import com.qmth.sop.common.enums.JobQuartzTypeEnum;
import com.qmth.sop.common.enums.JobTypeEnum;
import org.quartz.JobDetail;
import org.quartz.JobExecutionContext;
import org.quartz.JobExecutionException;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.scheduling.quartz.QuartzJobBean;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.HashMap;
import java.util.Map;
import java.util.Objects;

/**
 * @Description: 流程任务执行job
 * @Param:
 * @return:
 * @Author: wangliang
 * @Date: 2021/7/9
 */
public class FlowTaskQuartzJob extends QuartzJobBean {
    private final static Logger log = LoggerFactory.getLogger(FlowTaskQuartzJob.class);

    @Resource
    SmsSendUtil smsSendUtil;

    @Resource
    TSJobRemindService tsJobRemindService;

    @Override
    @Transactional
    protected void executeInternal(JobExecutionContext context) throws JobExecutionException {
        log.info("flowTaskQuartzJob,context:{}", context);
        try {
            JobDetail jobDetail = context.getJobDetail();
            String[] strings = jobDetail.getKey().toString().split("\\.");
            if (Objects.nonNull(strings[1])) {
                TSJobRemind tsJobRemind = tsJobRemindService.getById(Long.parseLong(strings[1]));
                if (Objects.nonNull(tsJobRemind)) {//数据还在则说明要发送提醒短信
                    if (tsJobRemind.getType() == JobQuartzTypeEnum.FLOW) {//流程发送短信
                        FlowTaskSmsResult flowTaskSmsResult = tsJobRemindService.getFlowTaskRemindSmsInfo(tsJobRemind.getId());
                        if (Objects.nonNull(flowTaskSmsResult)) {
                            String[] strs = tsJobRemind.getObjName().split(";");
                            Map<String, Object> templateParam = new HashMap<>();
                            templateParam.put("code", "test");
                            if (tsJobRemind.getRemindType() == JobTypeEnum.BEFORE) {
                                smsSendUtil.sendSms(flowTaskSmsResult.getMobileNumber(), SystemConstant.SMS_TPL_CODE, templateParam);
                            } else if (tsJobRemind.getRemindType() == JobTypeEnum.AFTER) {
                                smsSendUtil.sendSms(flowTaskSmsResult.getMobileNumber(), SystemConstant.SMS_TPL_CODE, templateParam);
                            }
                        }
                    } else if (tsJobRemind.getType() == JobQuartzTypeEnum.NOTICE) {//公告发送短信

                    }
                }
            }
        } catch (Exception e) {
            log.error(SystemConstant.LOG_ERROR, e);
        }
    }
}
